@@ -6,17 +6,17 @@
 #include <mysql/client_plugin.h>
 
 typedef struct st_mysql_client_plugin_AUTHENTICATION auth_plugin_t;
-static int client_mpvio_write_packet(struct st_plugin_vio*, const uchar*, size_t);
+static int client_mpvio_write_packet(struct st_plugin_vio*, const uchar*, int);
 static int native_password_auth_client(MYSQL_PLUGIN_VIO *vio, MYSQL *mysql);
 static int dummy_fallback_auth_client(MYSQL_PLUGIN_VIO *vio, MYSQL *mysql __attribute__((unused)));
 extern void read_user_name(char *name);
 extern char *ma_send_connect_attr(MYSQL *mysql, unsigned char *buffer);
 extern int ma_read_ok_packet(MYSQL *mysql, uchar *pos, ulong length);
-extern unsigned char *mysql_net_store_length(unsigned char *packet, size_t length);
+extern unsigned char *mysql_net_store_length(unsigned char *packet, ulonglong length);
 
 typedef struct {
   int (*read_packet)(struct st_plugin_vio *vio, uchar **buf);
-  int (*write_packet)(struct st_plugin_vio *vio, const uchar *pkt, size_t pkt_len);
+  int (*write_packet)(struct st_plugin_vio *vio, const uchar *pkt, int pkt_len);
   void (*info)(struct st_plugin_vio *vio, struct st_plugin_vio_info *info);
   /* -= end of MYSQL_PLUGIN_VIO =- */
   MYSQL *mysql;
@@ -120,7 +120,7 @@ static int dummy_fallback_auth_client(MYSQL_PLUGIN_VIO *vio, MYSQL *mysql __attr
   unsigned int i, last_errno= ((MCPVIO_EXT *)vio)->mysql->net.last_errno;
   if (last_errno)
   {
-    strncpy(last_error, ((MCPVIO_EXT *)vio)->mysql->net.last_error,
+    memcpy(last_error, ((MCPVIO_EXT *)vio)->mysql->net.last_error,
             sizeof(last_error) - 1);
     last_error[sizeof(last_error) - 1]= 0;
   }
@@ -137,7 +137,7 @@ static int dummy_fallback_auth_client(MYSQL_PLUGIN_VIO *vio, MYSQL *mysql __attr
   if (last_errno)
   {
     MYSQL *mysql= ((MCPVIO_EXT *)vio)->mysql;
-    strncpy(mysql->net.last_error, last_error,
+    memcpy(mysql->net.last_error, last_error,
             sizeof(mysql->net.last_error) - 1);
     mysql->net.last_error[sizeof(mysql->net.last_error) - 1]= 0;
   }
@@ -208,6 +208,11 @@ static int send_client_reply_packet(MCPVIO_EXT *mpvio,
   char *buff, *end;
   size_t conn_attr_len= (mysql->options.extension) ? 
                          mysql->options.extension->connect_attrs_len : 0;
+  size_t proxy_header_len= 0;
+  char *proxy_header=
+      (mysql->options.extension) ? mysql->options.extension->proxy_header : NULL;
+  if (proxy_header)
+    proxy_header_len= mysql->options.extension->proxy_header_len;
 
   /* see end= buff+32 below, fixed size of the packet is 32 bytes */
   buff= malloc(33 + USERNAME_LENGTH + data_len + NAME_LEN + NAME_LEN + conn_attr_len + 9);
@@ -263,12 +268,37 @@ static int send_client_reply_packet(MCPVIO_EXT *mpvio,
 
   /* Remove options that server doesn't support */
   mysql->client_flag= mysql->client_flag &
-                       (~(CLIENT_COMPRESS | CLIENT_SSL | CLIENT_PROTOCOL_41) 
+                       (~(CLIENT_COMPRESS | CLIENT_ZSTD_COMPRESSION | CLIENT_SSL | CLIENT_PROTOCOL_41) 
                        | mysql->server_capabilities);
 
-#ifndef HAVE_COMPRESS
-  mysql->client_flag&= ~CLIENT_COMPRESS;
-#endif
+  /* save compress for reconnect */
+  if (mysql->client_flag & CLIENT_COMPRESS)
+    mysql->options.compress= 1;
+
+  if (mysql->options.compress && (mysql->server_capabilities & CLIENT_COMPRESS))
+  {
+    /* For MySQL 8.0 we will use zstd compression */
+    if (mysql->server_capabilities & CLIENT_ZSTD_COMPRESSION)
+    {
+      if ((compression_plugin(net) = (MARIADB_COMPRESSION_PLUGIN *)mysql_client_find_plugin(mysql, 
+                                    _mariadb_compression_algorithm_str(COMPRESSION_ZSTD),
+                                    MARIADB_CLIENT_COMPRESSION_PLUGIN)))
+      {
+        mysql->client_flag|= CLIENT_ZSTD_COMPRESSION;
+        mysql->client_flag&= ~CLIENT_COMPRESS;
+      }
+    }
+    /* load zlib compression as default */
+    if (!compression_plugin(net))
+    {
+      if ((compression_plugin(net) = (MARIADB_COMPRESSION_PLUGIN *)mysql_client_find_plugin(mysql, 
+                                    _mariadb_compression_algorithm_str(COMPRESSION_ZLIB),
+                                    MARIADB_CLIENT_COMPRESSION_PLUGIN)))
+      {
+        mysql->client_flag|= CLIENT_COMPRESS;
+      }
+    }
+  }
 
   if (mysql->client_flag & CLIENT_PROTOCOL_41)
   {
@@ -315,6 +345,14 @@ static int send_client_reply_packet(MCPVIO_EXT *mpvio,
       Send mysql->client_flag, max_packet_size - unencrypted otherwise
       the server does not know we want to do SSL
     */
+    if (proxy_header_len)
+    {
+      ma_net_write_buff(net, proxy_header, proxy_header_len);
+      /* Reset proxy header */
+      proxy_header_len= 0;
+      proxy_header= NULL;
+    }
+
     if (ma_net_write(net, (unsigned char *)buff, (size_t) (end-buff)) || ma_net_flush(net))
     {
       my_set_error(mysql, CR_SERVER_LOST, SQLSTATE_UNKNOWN,
@@ -376,6 +414,24 @@ static int send_client_reply_packet(MCPVIO_EXT *mpvio,
 
   end= ma_send_connect_attr(mysql, (unsigned char *)end);
 
+  /* MySQL 8.0: 
+     If zstd compresson was specified, the server expects
+     1 byte for compression level
+  */
+  if (mysql->client_flag & CLIENT_ZSTD_COMPRESSION)
+  {
+    uchar compression_level= 3;
+    if (mysql->options.extension &&
+        mysql->options.extension->zstd_compression_level >= 1 &&
+        mysql->options.extension->zstd_compression_level <= 20)
+    {
+        compression_level= mysql->options.extension->zstd_compression_level;
+    }
+    *end++= compression_level;
+  }
+
+  if (proxy_header_len)
+    ma_net_write_buff(net, proxy_header, proxy_header_len);
   /* Write authentication package */
   if (ma_net_write(net, (unsigned char *)buff, (size_t) (end-buff)) || ma_net_flush(net))
   {
@@ -465,7 +521,7 @@ static int client_mpvio_read_packet(struct st_plugin_vio *mpv, uchar **buf)
 */
 
 static int client_mpvio_write_packet(struct st_plugin_vio *mpv,
-                                     const uchar *pkt, size_t pkt_len)
+                                     const uchar *pkt, int pkt_len)
 {
   int res;
   MCPVIO_EXT *mpvio= (MCPVIO_EXT*)mpv;
@@ -473,9 +529,9 @@ static int client_mpvio_write_packet(struct st_plugin_vio *mpv,
   if (mpvio->packets_written == 0)
   {
     if (mpvio->mysql_change_user)
-      res= send_change_user_packet(mpvio, pkt, (int)pkt_len);
+      res= send_change_user_packet(mpvio, pkt, pkt_len);
     else
-      res= send_client_reply_packet(mpvio, pkt, (int)pkt_len);
+      res= send_client_reply_packet(mpvio, pkt, pkt_len);
   }
   else
   {
@@ -580,6 +636,7 @@ int run_plugin_auth(MYSQL *mysql, char *data, uint data_len,
   ulong		pkt_length;
   int           res;
 
+
   /* determine the default/initial plugin to use */
   if (mysql->server_capabilities & CLIENT_PLUGIN_AUTH)
   {
@@ -622,6 +679,17 @@ int run_plugin_auth(MYSQL *mysql, char *data, uint data_len,
 retry:
   mpvio.plugin= auth_plugin;
 
+  if (auth_plugin_name &&
+     mysql->options.extension &&
+     mysql->options.extension->restricted_auth)
+  {
+    if (!strstr(mysql->options.extension->restricted_auth, auth_plugin_name))
+    {
+      my_set_error(mysql, CR_PLUGIN_NOT_ALLOWED, SQLSTATE_UNKNOWN, 0, data_plugin);
+      return 1;
+    }
+  }
+
   mysql->net.read_pos[0]= 0;
   res= auth_plugin->authenticate_user((struct st_plugin_vio *)&mpvio, mysql);
 
